/*******************************************************************************
 * Copyright (c) 2021, Bayerische Motoren Werke Aktiengesellschaft (BMW AG),
 *               2021 in-tech GmbH
 *
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *******************************************************************************/

#include "Condition.h"

namespace OPENSCENARIO
{

template <typename... Params>
ByAnyCondition toCondition(std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ICondition> condition, Params&&... args)
{
    if (auto by_entity_condition = condition->GetByEntityCondition(); by_entity_condition)
    {
        auto entity_condition = by_entity_condition->GetEntityCondition();
        auto triggering_entities = by_entity_condition->GetTriggeringEntities();
        if (!entity_condition || !triggering_entities)
        {
            throw std::runtime_error("Cannot specialize ICondition");
        }
        return toCondition(entity_condition, triggering_entities, std::forward<Params>(args)...);
    }
    if (auto by_value_condition = condition->GetByValueCondition(); by_value_condition)
    {
        return toCondition(by_value_condition, std::forward<Params>(args)...);
    }
    throw std::runtime_error("Cannot specialize ICondition");
}

Condition::Condition(std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ICondition> condition,
                     std::shared_ptr<mantle_api::IEnvironment> environment)
{
    const auto conditionEdge = condition->GetConditionEdge();
    evaluator_ = toEdgeEvaluator(conditionEdge);
    condition_ = toCondition(condition, environment);
}

bool Condition::IsSatisfied()
{
    auto is_satisfied = std::visit(Condition::EvaluateByAnyCondition{}, condition_);
    evaluator_->UpdateStatus(is_satisfied);
    return evaluator_->GetStatus();
}

}  // namespace OPENSCENARIO
