/*******************************************************************************
 * Copyright (c) 2021, Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
 *
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *******************************************************************************/

#include "builders/ActionBuilder.h"

#include <openScenarioLib/src/impl/NamedReferenceProxy.h>

namespace OPENSCENARIO::TESTING
{

FakeActionBuilder::FakeActionBuilder() : action_(std::make_shared<NET_ASAM_OPENSCENARIO::v1_1::ActionImpl>()) {}

FakeActionBuilder& FakeActionBuilder::WithPrivateAction(
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IPrivateActionWriter> private_action)
{
    action_->SetPrivateAction(private_action);
    return *this;
}

std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IActionWriter> FakeActionBuilder::Build()
{
    return action_;
}

FakePrivateActionBuilder::FakePrivateActionBuilder()
    : private_action_(std::make_shared<NET_ASAM_OPENSCENARIO::v1_1::PrivateActionImpl>())
{
}

FakePrivateActionBuilder& FakePrivateActionBuilder::WithTeleportAction(
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ITeleportActionWriter> teleport_action)
{
    private_action_->SetTeleportAction(teleport_action);
    return *this;
}

std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IPrivateActionWriter> FakePrivateActionBuilder::Build()
{
    return private_action_;
}

FakeTeleportActionBuilder::FakeTeleportActionBuilder()
    : teleport_action_(std::make_shared<NET_ASAM_OPENSCENARIO::v1_1::TeleportActionImpl>())
{
}

FakeTeleportActionBuilder& FakeTeleportActionBuilder::WithPosition(
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IPositionWriter> position)
{
    teleport_action_->SetPosition(position);
    return *this;
}

std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ITeleportActionWriter> FakeTeleportActionBuilder::Build()
{
    return teleport_action_;
}

FakeSpeedActionBuilder::FakeSpeedActionBuilder()
    : speed_action_(std::make_shared<NET_ASAM_OPENSCENARIO::v1_1::SpeedActionImpl>())
{
}

FakeSpeedActionBuilder& FakeSpeedActionBuilder::WithSpeedActionTarget(
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ISpeedActionTargetWriter> speed_action_target)
{
    speed_action_->SetSpeedActionTarget(speed_action_target);
    return *this;
}

FakeSpeedActionBuilder& FakeSpeedActionBuilder::WithTransitionDynamics(
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ITransitionDynamicsWriter> transition_dynamics)
{
    speed_action_->SetSpeedActionDynamics(transition_dynamics);
    return *this;
}

std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ISpeedActionWriter> FakeSpeedActionBuilder::Build()
{
    return speed_action_;
}

FakeSpeedActionTargetBuilder::FakeSpeedActionTargetBuilder()
    : speed_action_target_(std::make_shared<NET_ASAM_OPENSCENARIO::v1_1::SpeedActionTargetImpl>())
{
}

FakeSpeedActionTargetBuilder& FakeSpeedActionTargetBuilder::WithAbsoluteTargetSpeed(
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IAbsoluteTargetSpeedWriter> absolute_target_speed)
{
    speed_action_target_->SetAbsoluteTargetSpeed(absolute_target_speed);
    return *this;
}

FakeSpeedActionTargetBuilder& FakeSpeedActionTargetBuilder::WithRelativeTargetSpeed(
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IRelativeTargetSpeedWriter> relative_target_speed)
{
    speed_action_target_->SetRelativeTargetSpeed(relative_target_speed);
    return *this;
}

std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ISpeedActionTargetWriter> FakeSpeedActionTargetBuilder::Build()
{
    return speed_action_target_;
}

FakeAbsoluteTargetSpeedBuilder::FakeAbsoluteTargetSpeedBuilder(double value)
    : absolute_target_speed_(std::make_shared<NET_ASAM_OPENSCENARIO::v1_1::AbsoluteTargetSpeedImpl>())
{
    absolute_target_speed_->SetValue(value);
}

std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IAbsoluteTargetSpeedWriter> FakeAbsoluteTargetSpeedBuilder::Build()
{
    return absolute_target_speed_;
}

FakeRelativeTargetSpeedBuilder::FakeRelativeTargetSpeedBuilder(
    double value,
    std::string entity_name,
    const NET_ASAM_OPENSCENARIO::v1_1::SpeedTargetValueType::SpeedTargetValueTypeEnum speed_type)
    : relative_target_speed_(std::make_shared<NET_ASAM_OPENSCENARIO::v1_1::RelativeTargetSpeedImpl>())
{
    relative_target_speed_->SetValue(value);
    relative_target_speed_->SetSpeedTargetValueType(speed_type);
    auto relative_entity =
        std::make_shared<NET_ASAM_OPENSCENARIO::NamedReferenceProxy<NET_ASAM_OPENSCENARIO::v1_1::IEntity>>(entity_name);
    relative_target_speed_->SetEntityRef(relative_entity);
}

std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IRelativeTargetSpeedWriter> FakeRelativeTargetSpeedBuilder::Build()
{
    return relative_target_speed_;
}

FakeTransitionDynamicsBuilder::FakeTransitionDynamicsBuilder()
    : transition_dynamics_(std::make_shared<NET_ASAM_OPENSCENARIO::v1_1::TransitionDynamicsImpl>())
{
}

FakeTransitionDynamicsBuilder::FakeTransitionDynamicsBuilder(
    const NET_ASAM_OPENSCENARIO::v1_1::DynamicsShape::DynamicsShapeEnum dynamics_shape_enum,
    const NET_ASAM_OPENSCENARIO::v1_1::DynamicsDimension::DynamicsDimensionEnum dynamics_dimension_enum,
    double value)
    : transition_dynamics_(std::make_shared<NET_ASAM_OPENSCENARIO::v1_1::TransitionDynamicsImpl>())
{
    transition_dynamics_->SetDynamicsShape(dynamics_shape_enum);
    transition_dynamics_->SetDynamicsDimension(dynamics_dimension_enum);
    transition_dynamics_->SetValue(value);
}

FakeTransitionDynamicsBuilder& FakeTransitionDynamicsBuilder::WithDynamicsShape(
    NET_ASAM_OPENSCENARIO::v1_1::DynamicsShape dynamics_shape)
{
    transition_dynamics_->SetDynamicsShape(dynamics_shape);
    return *this;
}

FakeTransitionDynamicsBuilder& FakeTransitionDynamicsBuilder::WithDynamicsDimension(
    NET_ASAM_OPENSCENARIO::v1_1::DynamicsDimension dynamics_dimension)
{
    transition_dynamics_->SetDynamicsDimension(dynamics_dimension);
    return *this;
}

FakeTransitionDynamicsBuilder& FakeTransitionDynamicsBuilder::WithValue(double value)
{
    transition_dynamics_->SetValue(value);
    return *this;
}

std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ITransitionDynamicsWriter> FakeTransitionDynamicsBuilder::Build()
{
    return transition_dynamics_;
}

}  // namespace OPENSCENARIO::TESTING
