/*******************************************************************************
 * Copyright (c) 2021, Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
 *
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *******************************************************************************/

#pragma once
#include <openScenarioLib/generated/v1_1/impl/ApiClassImplV1_1.h>

#include <memory>

namespace OPENSCENARIO::TESTING
{

class FakeActionBuilder
{
  public:
    FakeActionBuilder();
    FakeActionBuilder& WithPrivateAction(
        std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IPrivateActionWriter> private_action);
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IActionWriter> Build();

  private:
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ActionImpl> action_{};
};

class FakePrivateActionBuilder
{
  public:
    FakePrivateActionBuilder();
    FakePrivateActionBuilder& WithTeleportAction(
        std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ITeleportActionWriter> teleport_action);
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IPrivateActionWriter> Build();

  private:
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::PrivateActionImpl> private_action_{};
};

class FakeTeleportActionBuilder
{
  public:
    FakeTeleportActionBuilder();
    FakeTeleportActionBuilder& WithPosition(std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IPositionWriter> position);
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ITeleportActionWriter> Build();

  private:
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::TeleportActionImpl> teleport_action_{};
};

class FakeSpeedActionBuilder
{
  public:
    FakeSpeedActionBuilder();
    FakeSpeedActionBuilder& WithSpeedActionTarget(
        std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ISpeedActionTargetWriter> speed_action_target);
    FakeSpeedActionBuilder& WithTransitionDynamics(
        std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ITransitionDynamicsWriter> transition_dynamics);
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ISpeedActionWriter> Build();

  private:
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::SpeedActionImpl> speed_action_{};
};

class FakeSpeedActionTargetBuilder
{
  public:
    FakeSpeedActionTargetBuilder();
    FakeSpeedActionTargetBuilder& WithAbsoluteTargetSpeed(
        std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IAbsoluteTargetSpeedWriter> absolute_target_speed);
    FakeSpeedActionTargetBuilder& WithRelativeTargetSpeed(
        std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IRelativeTargetSpeedWriter> relative_target_speed);
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ISpeedActionTargetWriter> Build();

  private:
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::SpeedActionTargetImpl> speed_action_target_{};
};

class FakeAbsoluteTargetSpeedBuilder
{
  public:
    FakeAbsoluteTargetSpeedBuilder(double value);
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IAbsoluteTargetSpeedWriter> Build();

  private:
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::AbsoluteTargetSpeedImpl> absolute_target_speed_{};
};

class FakeRelativeTargetSpeedBuilder
{
  public:
    FakeRelativeTargetSpeedBuilder(
        double value,
        std::string entity_name,
        const NET_ASAM_OPENSCENARIO::v1_1::SpeedTargetValueType::SpeedTargetValueTypeEnum speed_type);
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IRelativeTargetSpeedWriter> Build();

  private:
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::RelativeTargetSpeedImpl> relative_target_speed_{};
};

class FakeTransitionDynamicsBuilder
{
  public:
    FakeTransitionDynamicsBuilder();
    FakeTransitionDynamicsBuilder(
        const NET_ASAM_OPENSCENARIO::v1_1::DynamicsShape::DynamicsShapeEnum dynamics_shape_enum,
        const NET_ASAM_OPENSCENARIO::v1_1::DynamicsDimension::DynamicsDimensionEnum dynamics_dimension_enum,
        double value);
    FakeTransitionDynamicsBuilder& WithDynamicsShape(NET_ASAM_OPENSCENARIO::v1_1::DynamicsShape dynamics_shape);
    FakeTransitionDynamicsBuilder& WithDynamicsDimension(
        NET_ASAM_OPENSCENARIO::v1_1::DynamicsDimension dynamics_dimension);
    FakeTransitionDynamicsBuilder& WithValue(double value);
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ITransitionDynamicsWriter> Build();

  private:
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::TransitionDynamicsImpl> transition_dynamics_{};
};

}  // namespace OPENSCENARIO::TESTING
