/*******************************************************************************
 * Copyright (c) 2021, Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
 *
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *******************************************************************************/

#include "builders/PositionBuilder.h"

namespace OPENSCENARIO::TESTING
{

FakePositionBuilder::FakePositionBuilder() : position_(std::make_shared<NET_ASAM_OPENSCENARIO::v1_1::PositionImpl>()) {}

FakePositionBuilder& FakePositionBuilder::WithLanePosition(
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ILanePositionWriter> lane_position)
{
    position_->SetLanePosition(lane_position);
    return *this;
}

FakePositionBuilder& FakePositionBuilder::WithGeoPosition(
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IGeoPositionWriter> geo_position)
{
    position_->SetGeoPosition(geo_position);
    return *this;
}

std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IPositionWriter> FakePositionBuilder::Build()
{
    return position_;
}

FakeLanePositionBuilder::FakeLanePositionBuilder(std::string road_id, std::string lane_id, double offset, double s)
    : lane_position_(std::make_shared<NET_ASAM_OPENSCENARIO::v1_1::LanePositionImpl>())
{
    lane_position_->SetRoadId(road_id);
    lane_position_->SetLaneId(lane_id);
    lane_position_->SetOffset(offset);
    lane_position_->SetS(s);
}

FakeLanePositionBuilder& FakeLanePositionBuilder::WithOrientation(
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IOrientationWriter> orientation)
{
    lane_position_->SetOrientation(orientation);
    return *this;
}

std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::ILanePositionWriter> FakeLanePositionBuilder::Build()
{
    return lane_position_;
}

FakeGeoPositionBuilder::FakeGeoPositionBuilder(double latitude, double longitude)
    : geo_position_(std::make_shared<NET_ASAM_OPENSCENARIO::v1_1::GeoPositionImpl>())
{
    geo_position_->SetLatitude(latitude);
    geo_position_->SetLongitude(longitude);
}

FakeGeoPositionBuilder& FakeGeoPositionBuilder::WithOrientation(
    std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IOrientationWriter> orientation)
{
    geo_position_->SetOrientation(orientation);
    return *this;
}

std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_1::IGeoPositionWriter> FakeGeoPositionBuilder::Build()
{
    return geo_position_;
}

}  // namespace OPENSCENARIO::TESTING
